/*
 * This dll is part of Sunshine's Desktop Symbol Utility.
 * It's injected into explorer.exe and subclasses the listview window procedure
 * to change text and background colors of the symbols.
 */
#include <windows.h>
#include <commctrl.h>
#include "shared.h"


#pragma data_seg (".shared")
COLORREF textColor = RGB(0, 0, 0);
BOOL	 bActivate = FALSE;				// is set from the loader, don't modify here.
										// see my notes on my website for further details
COLORREF textBgColor = RGB(255,255, 255);	
#pragma data_seg ()

#pragma comment(linker,"/SECTION:.shared,RWS") 
#pragma comment(linker, "/NODEFAULTLIB")
#pragma comment(linker,"/ENTRY:DllMain")



// Global variables
WNDPROC		oldproc;				// address of the original window procedure
COLORREF	oldTextColor;			// save original text color
COLORREF	oldTextBgColor;			// save original text background color


/*
 * Registry helper function
 */
BOOL GetRegDword(LPSTR szKeyName, LPSTR szValueName, LPDWORD retValue)
{
	HKEY		hKey;
	DWORD		dwSize;
	DWORD		dwType;

	if (RegOpenKeyEx(HKEY_CURRENT_USER, szKeyName, 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
	{
		if (RegQueryValueEx(hKey, szValueName, 0, &dwType, (LPBYTE)retValue, &dwSize) == ERROR_SUCCESS)
		{
			RegCloseKey(hKey);
			return TRUE;
		}
	}
	return FALSE;
}

/*
 * Try to load saved color values from registry
 */
void GetColorsFromReg()
{
	LPSTR	key = "Software\\Sunshine2k\\DeskSym";
	LPSTR	value = "TextColor";

	if (!GetRegDword(key, value, &textColor))
		textColor = RGB(0,0,0);
	value = "TextBgColor";
	if (!GetRegDword(key, value, &textBgColor))
		textBgColor = RGB(255,255,255);
	return;
}



/*
 * new window proc
 */
LRESULT CALLBACK SubClassProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{

	case WM_PAINT:
		SendMessage(hwnd, LVM_SETTEXTCOLOR, 0, textColor);
		SendMessage(hwnd, LVM_SETTEXTBKCOLOR, 0, textBgColor);

	default:break;

	}

	return CallWindowProc(oldproc, hwnd, msg, wParam, lParam);
}




/*
 * Dll Entrypoint
 */

BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD nReason, LPVOID pReserved)
{
	
	// ** process attach
	if(nReason == DLL_PROCESS_ATTACH)
	{
		if (!bActivate) return TRUE;

		// find desktop listview window
		HWND hDesktop = FindWindowEx(FindWindow("Progman", 0), 0, "SHELLDLL_DefView", 0);
		HWND hListView = FindWindowEx(hDesktop, 0, "SysListView32", 0);

		// save original colors
		oldTextColor = (COLORREF)SendMessage(hListView, LVM_GETTEXTCOLOR, 0, 0);
		oldTextBgColor = (COLORREF)SendMessage(hListView, LVM_GETTEXTBKCOLOR, 0, 0);

		// load saved values from registry
		GetColorsFromReg();

		// then subclass it
		oldproc = 0;
		oldproc = (WNDPROC)SetWindowLong(hListView, GWL_WNDPROC, (LONG)SubClassProc);
		
		InvalidateRect(hListView, NULL, TRUE);

		bActivate = FALSE;
	}

	// ** dll is about to being unloaded
	if (nReason == DLL_PROCESS_DETACH)
	{
		if (!bActivate) return TRUE;

		// find listview window
		HWND hDesktop = FindWindowEx(FindWindow("Progman", 0), 0, "SHELLDLL_DefView", 0);
		HWND hListView = FindWindowEx(hDesktop, 0, "SysListView32", 0);
		if (oldproc != 0)
			SetWindowLong(hListView, GWL_WNDPROC, (LONG)oldproc);

		// restore original colors
		SendMessage(hListView, LVM_SETTEXTCOLOR, 0, oldTextColor);
		SendMessage(hListView, LVM_SETTEXTBKCOLOR, 0, oldTextBgColor);
		InvalidateRect(hListView, NULL, TRUE);

		bActivate = FALSE;
	}

	// Return success
	return TRUE;
}